@@ -35,10 +35,6 @@
 (require 'elmo)
 (require 'elmo-map)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; cache backend by Kenichi OKADA <okada@opaopa.org>
-;;
 (eval-and-compile
   (luna-define-class elmo-cache-folder (elmo-map-folder) (dir-name directory))
   (luna-define-internal-accessors 'elmo-cache-folder))
@@ -135,14 +131,18 @@
 					       &optional number)
   ;; dir-name is changed according to msgid.
   (unless (elmo-cache-folder-dir-name-internal folder)
-    (let* ((file (elmo-file-cache-get-path (std11-field-body "message-id")))
-	   (dir (directory-file-name (file-name-directory file))))
-      (unless (file-exists-p dir)
-	(elmo-make-directory dir))
-      (when (file-writable-p file)
-	(write-region-as-binary
-	 (point-min) (point-max) file nil 'no-msg))))
-  t)
+    (let ((msgid (std11-field-body "message-id"))
+	  file dir)
+      (when msgid
+	(setq file (elmo-file-cache-get-path msgid))
+	(setq dir (directory-file-name (file-name-directory file)))
+	(unless (file-exists-p dir)
+	  (elmo-make-directory dir))
+	(when (and (file-writable-p file)
+		   (not (file-exists-p file)))
+	  (write-region-as-binary
+	   (point-min) (point-max) file nil 'no-msg)
+	  t)))))
 
 (luna-define-method elmo-map-folder-delete-messages ((folder elmo-cache-folder)
 						     locations)
@@ -165,38 +165,12 @@
     (when (file-exists-p file)
       (insert-file-contents-as-binary file))))
 
-(luna-define-method elmo-folder-creatable-p ((folder elmo-cache-folder))
-  nil)
-
 (luna-define-method elmo-folder-writable-p ((folder elmo-cache-folder))
   t)
 
 (luna-define-method elmo-folder-exists-p ((folder elmo-cache-folder))
   t)
 
-(luna-define-method elmo-folder-search ((folder elmo-cache-folder)
-					condition &optional from-msgs)
-  (let* ((msgs (or from-msgs (elmo-folder-list-messages folder)))
-	 (number-list msgs)
-	 (i 0)
-	 (num (length msgs))
-	 file
-	 matched
-	 case-fold-search)
-    (while msgs
-      (if (and (setq file (elmo-message-file-name folder (car msgs)))
-	       (file-exists-p file)
-	       (elmo-file-field-condition-match file
-						condition
-						(car msgs)
-						number-list))
-	  (setq matched (nconc matched (list (car msgs)))))
-      (elmo-display-progress
-       'elmo-internal-folder-search "Searching..."
-       (/ (* (setq i (1+ i)) 100) num))
-      (setq msgs (cdr msgs)))
-    matched))
-
 (luna-define-method elmo-message-file-p ((folder elmo-cache-folder) number)
   t)
 
