@@ -35,6 +35,11 @@
 (require 'elmo)
 (require 'elmo-map)
 
+(defcustom elmo-maildir-folder-path "~/Maildir"
+  "*Maildir folder path."
+  :type 'directory
+  :group 'elmo)
+
 ;;; ELMO Maildir folder
 (eval-and-compile
   (luna-define-class elmo-maildir-folder
@@ -209,7 +214,10 @@ LOCATION."
     (while news
       (rename-file
        (expand-file-name (car news) (expand-file-name "new" maildir))
-       (expand-file-name (concat (car news) ":2,")
+       (expand-file-name (concat
+			  (car news)
+			  (unless (string-match ":2,[A-Z]*$" (car news))
+			    ":2,"))
 			 (expand-file-name "cur" maildir)))
       (setq news (cdr news)))))
 
@@ -457,43 +465,44 @@ file name for maildir directories."
 	  t)
       (error))))
 
-(luna-define-method elmo-folder-delete :before ((folder elmo-maildir-folder))
-  (let ((basedir (elmo-maildir-folder-directory-internal folder)))
-    (condition-case nil
-	(let ((tmp-files (directory-files
-			  (expand-file-name "tmp" basedir)
-			  t "[^.].*")))
-	  ;; Delete files in tmp.
-	  (dolist (file tmp-files)
-	    (delete-file file))
-	  (dolist (dir '("new" "cur" "tmp" "."))
-	    (setq dir (expand-file-name dir basedir))
-	    (if (not (file-directory-p dir))
-		(error nil)
-	      (elmo-delete-directory dir t)))
-	  t)
-      (error nil))))
+(luna-define-method elmo-folder-delete ((folder elmo-maildir-folder))
+  (let ((msgs (and (elmo-folder-exists-p folder)
+		   (elmo-folder-list-messages folder))))
+    (when (yes-or-no-p (format "%sDelete msgdb and substance of \"%s\"? "
+			       (if (> (length msgs) 0)
+				   (format "%d msg(s) exists. " (length msgs))
+				 "")
+			       (elmo-folder-name-internal folder)))
+      (let ((basedir (elmo-maildir-folder-directory-internal folder)))
+	(condition-case nil
+	    (let ((tmp-files (directory-files
+			      (expand-file-name "tmp" basedir)
+			      t "[^.].*")))
+	      ;; Delete files in tmp.
+	      (dolist (file tmp-files)
+		(delete-file file))
+	      (dolist (dir '("new" "cur" "tmp" "."))
+		(setq dir (expand-file-name dir basedir))
+		(if (not (file-directory-p dir))
+		    (error nil)
+		  (elmo-delete-directory dir t))))
+	  (error nil)))
+      (elmo-msgdb-delete-path folder)
+      t)))
 
-(luna-define-method elmo-folder-search ((folder elmo-maildir-folder)
-					condition &optional numbers)
-  (save-excursion
-    (let* ((msgs (or numbers (elmo-folder-list-messages folder)))
-	   (i 0)
-	   case-fold-search matches
-	   percent num
-	   (len (length msgs))
-	   number-list msg-num)
-      (setq number-list msgs)
-      (dolist (number numbers)
-	(if (elmo-file-field-condition-match
-	     (elmo-message-file-name folder number)
-	     condition number number-list)
-	    (setq matches (cons number matches)))
-	(setq i (1+ i))
-	(elmo-display-progress
-	 'elmo-maildir-search "Searching..."
-	 (/ (* i 100) len)))
-      (nreverse matches))))
+(luna-define-method elmo-folder-rename-internal ((folder elmo-maildir-folder)
+						 new-folder)
+  (let* ((old (elmo-maildir-folder-directory-internal folder))
+	 (new (elmo-maildir-folder-directory-internal new-folder))
+	 (new-dir (directory-file-name (file-name-directory new))))
+    (unless (file-directory-p old)
+      (error "No such directory: %s" old))
+    (when (file-exists-p new)
+      (error "Already exists directory: %s" new))
+    (unless (file-directory-p new-dir)
+      (elmo-make-directory new-dir))
+    (rename-file old new)
+    t))
 
 (require 'product)
 (product-provide (provide 'elmo-maildir) (require 'elmo-version))
