@@ -208,48 +208,6 @@ content of MSGDB is changed."
 	  (elmo-msgdb-append-element ret-val (cons number id)))
     ret-val))
 
-;;;
-;; parsistent mark handling
-;; (for global!)
-
-(defvar elmo-msgdb-global-mark-alist nil)
-
-(defun elmo-msgdb-global-mark-delete (msgid)
-  (let* ((path (expand-file-name
-		elmo-msgdb-global-mark-filename
-		elmo-msgdb-directory))
-	 (malist (or elmo-msgdb-global-mark-alist
-		     (setq elmo-msgdb-global-mark-alist
-			   (elmo-object-load path))))
-	 match)
-    (when (setq match (assoc msgid malist))
-      (setq elmo-msgdb-global-mark-alist
-	    (delete match elmo-msgdb-global-mark-alist))
-      (elmo-object-save path elmo-msgdb-global-mark-alist))))
-
-(defun elmo-msgdb-global-mark-set (msgid mark)
-  (let* ((path (expand-file-name
-		elmo-msgdb-global-mark-filename
-		elmo-msgdb-directory))
-	 (malist (or elmo-msgdb-global-mark-alist
-		     (setq elmo-msgdb-global-mark-alist
-			   (elmo-object-load path))))
-	 match)
-    (if (setq match (assoc msgid malist))
-	(setcdr match mark)
-      (setq elmo-msgdb-global-mark-alist
-	    (nconc elmo-msgdb-global-mark-alist
-		   (list (cons msgid mark)))))
-    (elmo-object-save path elmo-msgdb-global-mark-alist)))
-
-(defun elmo-msgdb-global-mark-get (msgid)
-  (cdr (assoc msgid (or elmo-msgdb-global-mark-alist
-			(setq elmo-msgdb-global-mark-alist
-			      (elmo-object-load
-			       (expand-file-name
-				elmo-msgdb-global-mark-filename
-				elmo-msgdb-directory)))))))
-
 ;;;
 ;; persistent mark handling
 ;; (for each folder)
@@ -611,12 +569,14 @@ header separator."
   (and entity (car entity)))
 
 (defsubst elmo-msgdb-overview-entity-get-extra-field (entity field-name)
-  (let ((extra (and entity (aref (cdr entity) 8))))
+  (let ((field-name (downcase field-name))
+	(extra (and entity (aref (cdr entity) 8))))
     (and extra
 	 (cdr (assoc field-name extra)))))
 
 (defsubst elmo-msgdb-overview-entity-set-extra-field (entity field-name value)
-  (let ((extras (and entity (aref (cdr entity) 8)))
+  (let ((field-name (downcase field-name))
+	(extras (and entity (aref (cdr entity) 8)))
 	extra)
     (if (setq extra (assoc field-name extras))
 	(setcdr extra value)
@@ -768,12 +728,16 @@ header separator."
     seen-list))
 
 (defun elmo-msgdb-get-message-id-from-buffer ()
-  (or (elmo-field-body "message-id")
+  (let ((msgid (elmo-field-body "message-id")))
+    (if msgid
+	(if (string-match "<\\(.+\\)>$" msgid)
+	    msgid
+	  (concat "<" msgid ">")) ; Invaild message-id.
       ;; no message-id, so put dummy msgid.
       (concat "<" (timezone-make-date-sortable
 		   (elmo-field-body "date"))
 	      (nth 1 (eword-extract-address-components
-		      (or (elmo-field-body "from") "nobody"))) ">")))
+		      (or (elmo-field-body "from") "nobody"))) ">"))))
 
 (defsubst elmo-msgdb-create-overview-from-buffer (number &optional size time)
   "Create overview entity from current buffer.
