@@ -237,7 +237,7 @@
 	(elmo-folder-set-msgdb-internal folder
 					(elmo-msgdb-append
 					 (elmo-folder-msgdb folder)
-					 append-msgdb t))
+					 append-msgdb))
 	(length to-be-deleted))
     0))
 
@@ -319,7 +319,9 @@
 				      numbers))))
     (while flds
       (setq nums (elmo-folder-diff (car flds) (car num-list))
-	    nums (cons (elmo-diff-unread nums) (elmo-diff-all nums)))
+	    nums (cons (or (elmo-diff-unread nums)
+			   (elmo-diff-new nums))
+		       (elmo-diff-all nums)))
       (setq diffs (nconc diffs (list nums)))
       (setq count (+ 1 count))
       (setq num-list (cdr num-list))
