@@ -75,8 +75,6 @@
     (filename newname &optional ok-if-already-exists)
     (copy-file filename newname ok-if-already-exists t)))
 
-(defalias 'elmo-read 'read)
-
 (defmacro elmo-set-work-buf (&rest body)
   "Execute BODY on work buffer.  Work buffer remains."
   (` (save-excursion
@@ -132,7 +130,8 @@ File content is encoded with MIME-CHARSET."
 Directory of the file is created if it doesn't exist.
 File content is encoded with MIME-CHARSET."
   (elmo-set-work-buf
-   (prin1 object (current-buffer))
+   (let (print-length print-level)
+     (prin1 object (current-buffer)))
 ;;;(princ "\n" (current-buffer))
    (elmo-save-buffer filename mime-charset)))
 
@@ -269,7 +268,7 @@ Return value is a cons cell of (STRUCTURE . REST)"
 (defun elmo-condition-parse-search-value ()
   (cond
    ((looking-at "\"")
-    (elmo-read (current-buffer)))
+    (read (current-buffer)))
    ((or (looking-at "yesterday") (looking-at "lastweek")
 	(looking-at "lastmonth") (looking-at "lastyear")
 	(looking-at "[0-9]+ *daysago")
@@ -417,7 +416,8 @@ Return value is a cons cell of (STRUCTURE . REST)"
   (save-excursion
     (let ((filename (expand-file-name elmo-passwd-alist-file-name
 				      elmo-msgdb-directory))
-	  (tmp-buffer (get-buffer-create " *elmo-passwd-alist-tmp*")))
+	  (tmp-buffer (get-buffer-create " *elmo-passwd-alist-tmp*"))
+	  print-length print-level)
       (set-buffer tmp-buffer)
       (erase-buffer)
       (prin1 elmo-passwd-alist tmp-buffer)
@@ -765,7 +765,7 @@ the directory becomes empty after deletion."
 
 (defun elmo-list-diff (list1 list2 &optional mes)
   (if mes
-      (message mes))
+      (message "%s" mes))
   (let ((clist1 (copy-sequence list1))
 	(clist2 (copy-sequence list2)))
     (while list2
@@ -775,7 +775,7 @@ the directory becomes empty after deletion."
       (setq clist2 (delq (car list1) clist2))
       (setq list1 (cdr list1)))
     (if mes
-	(message (concat mes "done.")))
+	(message "%sdone" mes))
     (list clist1 clist2)))
 
 (defun elmo-list-bigger-diff (list1 list2 &optional mes)
@@ -1272,6 +1272,25 @@ But if optional argument AUTO is non-nil, DEFAULT is returned."
 	  (throw 'found t))
       (setq slist (cdr slist)))))
 
+(cond ((fboundp 'member-ignore-case)
+       (defalias 'elmo-string-member-ignore-case 'member-ignore-case))
+      ((fboundp 'compare-strings)
+       (defun elmo-string-member-ignore-case (elt list)
+	 "Like `member', but ignores differences in case and text representation.
+ELT must be a string.  Upper-case and lower-case letters are treated as equal.
+Unibyte strings are converted to multibyte for comparison."
+	 (while (and list (not (eq t (compare-strings elt 0 nil (car list) 0 nil t))))
+	   (setq list (cdr list)))
+	 list))
+      (t
+       (defun elmo-string-member-ignore-case (elt list)
+	 "Like `member', but ignores differences in case and text representation.
+ELT must be a string.  Upper-case and lower-case letters are treated as equal."
+	 (let ((str (downcase elt)))
+	   (while (and list (not (string= str (downcase (car list)))))
+	     (setq list (cdr list)))
+	   list))))
+
 (defun elmo-string-match-member (str list &optional case-ignore)
   (let ((case-fold-search case-ignore))
     (catch 'member
@@ -1563,6 +1582,49 @@ NUMBER-SET is altered."
 				  (match-end matchn)) list)))
     (nreverse list)))
 
+;;;
+;; parsistent mark handling
+;; (for global!)
+;; (FIXME: this should be treated in the msgdb.)
+
+(defvar elmo-msgdb-global-mark-alist nil)
+
+(defun elmo-msgdb-global-mark-delete (msgid)
+  (let* ((path (expand-file-name
+		elmo-msgdb-global-mark-filename
+		elmo-msgdb-directory))
+	 (malist (or elmo-msgdb-global-mark-alist
+		     (setq elmo-msgdb-global-mark-alist
+			   (elmo-object-load path))))
+	 match)
+    (when (setq match (assoc msgid malist))
+      (setq elmo-msgdb-global-mark-alist
+	    (delete match elmo-msgdb-global-mark-alist))
+      (elmo-object-save path elmo-msgdb-global-mark-alist))))
+
+(defun elmo-msgdb-global-mark-set (msgid mark)
+  (let* ((path (expand-file-name
+		elmo-msgdb-global-mark-filename
+		elmo-msgdb-directory))
+	 (malist (or elmo-msgdb-global-mark-alist
+		     (setq elmo-msgdb-global-mark-alist
+			   (elmo-object-load path))))
+	 match)
+    (if (setq match (assoc msgid malist))
+	(setcdr match mark)
+      (setq elmo-msgdb-global-mark-alist
+	    (nconc elmo-msgdb-global-mark-alist
+		   (list (cons msgid mark)))))
+    (elmo-object-save path elmo-msgdb-global-mark-alist)))
+
+(defun elmo-msgdb-global-mark-get (msgid)
+  (cdr (assoc msgid (or elmo-msgdb-global-mark-alist
+			(setq elmo-msgdb-global-mark-alist
+			      (elmo-object-load
+			       (expand-file-name
+				elmo-msgdb-global-mark-filename
+				elmo-msgdb-directory)))))))
+
 ;;; File cache.
 (defmacro elmo-make-file-cache (path status)
   "PATH is the cache file name.
@@ -1619,14 +1681,16 @@ SECTION is the section string."
 
 (defun elmo-file-cache-delete (path)
   "Delete a cache on PATH."
-  (when (file-exists-p path)
-    (if (file-directory-p path)
-	(progn
-	  (dolist (file (directory-files path t "^[^\\.]"))
-	    (delete-file file))
-	  (delete-directory path))
-      (delete-file path))
-    t))
+  (unless (elmo-msgdb-global-mark-get
+	   (elmo-cache-to-msgid (file-name-nondirectory path)))
+    (when (file-exists-p path)
+      (if (file-directory-p path)
+	  (progn
+	    (dolist (file (directory-files path t "^[^\\.]"))
+	      (delete-file file))
+	    (delete-directory path))
+	(delete-file path))
+      t)))
 
 (defun elmo-file-cache-exists-p (msgid)
   "Returns 'section or 'entire if a cache which corresponds to MSGID exists."
