@@ -29,7 +29,7 @@
 ;;; Code:
 
 (defconst wl-demo-copyright-notice
-  "Copyright (C) 1998-2001 Yuuichi Teranishi <teranisi@gohome.org>"
+  "Copyright (C) 1998-2004 Yuuichi Teranishi <teranisi@gohome.org>"
   "A declaration of the copyright on Wanderlust.")
 
 (eval-when-compile
@@ -39,13 +39,12 @@
 (require 'wl-version)
 (require 'wl-highlight)
 
-(defconst wl-demo-icon-name
-  (concat "wl-" (wl-version-status)
-	  (if (string-match "^... Dec \\([ 1][0-9]\\|2[0-5]\\)"
-			    (current-time-string))
-	      "-xmas-logo"
-	    "-logo"))
-  "Basename of the logo file.")
+(defun wl-demo-icon-name ()
+  "A function to determine logo file name."
+  (catch 'found
+    (dolist (pair wl-demo-icon-name-alist)
+      (when (eval (car pair))
+	(throw 'found (eval (cdr pair)))))))
 
 (defvar wl-logo-ascii "\
         o$                  oo$$$$$$ooo
@@ -79,6 +78,7 @@ Yet Another Message Interface On Emacsen"
   (autoload 'make-glyph "glyphs")
   (autoload 'set-glyph-face "glyphs")
   (autoload 'set-specifier "specifier")
+  (defalias-maybe 'face-background-name 'ignore)
   (defalias-maybe 'frame-char-height 'ignore)
   (defalias-maybe 'frame-char-width 'ignore)
   (defalias-maybe 'glyph-height 'ignore)
@@ -134,18 +134,19 @@ Yet Another Message Interface On Emacsen"
 IMAGE-TYPE specifies what a type of image should be displayed.
 Return a number of lines that an image occupies in the buffer."
   (let ((file (cond ((eq 'xpm image-type)
-		     (concat wl-demo-icon-name ".xpm"))
+		     (concat (wl-demo-icon-name) ".xpm"))
 		    ((eq 'bitmap image-type)
-		     (concat wl-demo-icon-name ".img"))
+		     (concat (wl-demo-icon-name) ".img"))
 		    ((eq 'xbm image-type)
-		     (concat wl-demo-icon-name ".xbm"))))
+		     (concat (wl-demo-icon-name) ".xbm"))))
 	image width height)
     (when (featurep 'xemacs)
       (when (boundp 'default-gutter-visible-p)
 	(set-specifier (symbol-value 'default-gutter-visible-p)
 		       nil (current-buffer)))
-      (set-specifier (symbol-value 'scrollbar-height) 0 (current-buffer))
-      (set-specifier (symbol-value 'scrollbar-width) 0 (current-buffer)))
+      (when (featurep 'scrollbar)
+	(set-specifier (symbol-value 'scrollbar-height) 0 (current-buffer))
+	(set-specifier (symbol-value 'scrollbar-width) 0 (current-buffer))))
     (if (and file
 	     (if (and wl-icon-directory
 		      (file-directory-p wl-icon-directory))
