@@ -1,6 +1,7 @@
 ;;; wl-highlight.el --- Hilight modules for Wanderlust.
 
-;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+;;  Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;; Keywords: mail, net news
@@ -42,12 +43,6 @@
 	 (require 'wl-e21))
 	(t
 	 (require 'wl-mule)))
-  (defun-maybe extent-begin-glyph (a))
-  (defun-maybe delete-extent (a))
-  (defun-maybe make-extent (a b))
-  (defun-maybe set-extent-begin-glyph (a b))
-  (defun-maybe set-extent-end-glyph (a b))
-  (defun-maybe extent-at (a b c d e))
   (defun-maybe wl-dnd-set-drop-target (a b))
   (defun-maybe wl-dnd-set-drag-starter (a b)))
 
@@ -776,12 +771,12 @@
 
 (defun wl-highlight-summary-line-string (line mark temp-mark indent)
   (let (fsymbol)
-    (cond ((and (string= temp-mark "+")
+    (cond ((and (string= temp-mark wl-summary-score-over-mark)
 		(member mark (list wl-summary-unread-cached-mark
 				   wl-summary-unread-uncached-mark
 				   wl-summary-new-mark)))
 	   (setq fsymbol 'wl-highlight-summary-high-unread-face))
-	  ((and (string= temp-mark "-")
+	  ((and (string= temp-mark wl-summary-score-below-mark)
 		(member mark (list wl-summary-unread-cached-mark
 				   wl-summary-unread-uncached-mark
 				   wl-summary-new-mark)))
@@ -801,9 +796,9 @@
 	   (setq fsymbol 'wl-highlight-summary-unread-face))
 	  ((or (string= mark wl-summary-important-mark))
 	   (setq fsymbol 'wl-highlight-summary-important-face))
-	  ((string= temp-mark "-")
+	  ((string= temp-mark wl-summary-score-below-mark)
 	   (setq fsymbol 'wl-highlight-summary-low-read-face))
-	  ((string= temp-mark "+")
+	  ((string= temp-mark wl-summary-score-over-mark)
 	   (setq fsymbol 'wl-highlight-summary-high-read-face))
 	  (t (if (zerop (length indent))
 		 (setq fsymbol 'wl-highlight-summary-thread-top-face)
@@ -818,48 +813,35 @@
     (let ((inhibit-read-only t)
 	  (case-fold-search nil) temp-mark status-mark
 	  (deactivate-mark nil)
-	  (sregexp (concat
-		    "^"
-		    wl-summary-buffer-number-regexp
-		    "\\(.\\)\\(.\\)../..\(.*\)..:.. \\("
-		    wl-highlight-thread-indent-string-regexp
-		    "\\)[[<]"))
-	  fregexp fsymbol bol eol matched thread-top looked-at dest ds)
+	  fsymbol bol eol matched thread-top looked-at dest ds)
       (end-of-line)
       (setq eol (point))
       (beginning-of-line)
       (setq bol (point))
       (if smark
 	  (setq status-mark smark)
-	(setq looked-at (looking-at sregexp))
-	(when looked-at
-	  (setq status-mark (buffer-substring (match-beginning 2)
-					      (match-end 2)))))
+	(setq status-mark (wl-summary-persistent-mark)))
       (when temp-too
-	(unless looked-at
-	  (setq looked-at (looking-at sregexp)))
-	(when looked-at
-	  (setq temp-mark (buffer-substring (match-beginning 1)
-					    (match-end 1)))
-	  (cond
-	   ((string= temp-mark "*")
-	    (setq fsymbol 'wl-highlight-summary-temp-face))
-	   ((string= temp-mark "D")
-	    (setq fsymbol 'wl-highlight-summary-deleted-face))
-	   ((string= temp-mark "O")
-	    (setq fsymbol 'wl-highlight-summary-copied-face
-		  dest t))
-	   ((string= temp-mark "o")
-	    (setq fsymbol 'wl-highlight-summary-refiled-face
-		  dest t)))))
+	(setq temp-mark (wl-summary-temp-mark))
+	(cond
+	 ((string= temp-mark "*")
+	  (setq fsymbol 'wl-highlight-summary-temp-face))
+	 ((string= temp-mark "D")
+	  (setq fsymbol 'wl-highlight-summary-deleted-face))
+	 ((string= temp-mark "O")
+	  (setq fsymbol 'wl-highlight-summary-copied-face
+		dest t))
+	 ((string= temp-mark "o")
+	  (setq fsymbol 'wl-highlight-summary-refiled-face
+		dest t))))
       (if (not fsymbol)
 	  (cond
-	   ((and (string= temp-mark "+")
+	   ((and (string= temp-mark wl-summary-score-over-mark)
 		 (member status-mark (list wl-summary-unread-cached-mark
 					   wl-summary-unread-uncached-mark
 					   wl-summary-new-mark)))
 	    (setq fsymbol 'wl-highlight-summary-high-unread-face))
-	   ((and (string= temp-mark "-")
+	   ((and (string= temp-mark wl-summary-score-below-mark)
 		 (member status-mark (list wl-summary-unread-cached-mark
 					   wl-summary-unread-uncached-mark
 					   wl-summary-new-mark)))
@@ -872,15 +854,14 @@
 	   ((string= status-mark wl-summary-important-mark)
 	    (setq fsymbol 'wl-highlight-summary-important-face))
 	   ;; score mark
-	   ((string= temp-mark "-")
+	   ((string= temp-mark wl-summary-score-below-mark)
 	    (setq fsymbol 'wl-highlight-summary-low-read-face))
-	   ((string= temp-mark "+")
+	   ((string= temp-mark wl-summary-score-over-mark)
 	    (setq fsymbol 'wl-highlight-summary-high-read-face))
 	   ;;
-	   (t (if (and looked-at
-		       (string= (buffer-substring
-				 (match-beginning 3)
-				 (match-end 3)) ""))
+	   (t (if (null
+		   (wl-thread-entity-get-parent-entity
+		    (wl-thread-get-entity (wl-summary-message-number))))
 		  (setq fsymbol 'wl-highlight-summary-thread-top-face)
 		(setq fsymbol 'wl-highlight-summary-normal-face)))))
       (put-text-property bol eol 'face fsymbol)
@@ -968,7 +949,7 @@ Variables used:
   (interactive)
   (wl-highlight-summary (point-min)(point-max)))
 
-(defun wl-highlight-summary (start end)
+(defun wl-highlight-summary (start end &optional lazy)
   "Highlight summary between start and end.
 Faces used:
   wl-highlight-summary-unread-face      unread messages
@@ -998,17 +979,11 @@ Variables used:
       (setq i 0)
       (while (and (not (eobp))
 		  (< (point) end))
-	(wl-highlight-summary-current-line nil nil
-					   (or wl-summary-lazy-highlight
-					       wl-summary-scored))
-	(when (and (not wl-summary-lazy-highlight)
-		   (> lines elmo-display-progress-threshold))
-	  (setq i (+ i 1))
-	  (setq percent (/ (* i 100) lines))
-	  (if (or (zerop (% percent 5)) (= i lines))
-	      (elmo-display-progress
-	       'wl-highlight-summary "Highlighting..."
-	       percent)))
+	(when (or (not lazy)
+		  (null (get-text-property (point) 'face)))
+	  (wl-highlight-summary-current-line nil nil
+					     (or wl-summary-lazy-highlight
+						 wl-summary-scored)))
 	(forward-line 1))
       (unless wl-summary-lazy-highlight
 	(message "Highlighting...done")))))
@@ -1016,14 +991,18 @@ Variables used:
 (defun wl-highlight-summary-window (&optional win beg)
   "Highlight summary window.
 This function is defined for `window-scroll-functions'"
-  (if wl-summary-highlight
-      (with-current-buffer (window-buffer win)
-	(wl-highlight-summary (window-start win)
-			      (save-excursion
-				(goto-char (window-start win))
-				(forward-line (frame-height))
-				(point)))
-	(set-buffer-modified-p nil))))
+  (when wl-summary-highlight
+    (with-current-buffer (window-buffer win)
+      (when (eq major-mode 'wl-summary-mode)
+	(let ((start (window-start win))
+	      (end (condition-case nil
+		       (window-end win t) ;; old emacsen doesn't support 2nd arg.
+		     (error (window-end win)))))
+	  (wl-highlight-summary start
+				end
+				'lazy))
+	(set-buffer-modified-p nil)))))
+
 
 (defun wl-highlight-headers (&optional for-draft)
   (let ((beg (point-min))
@@ -1124,8 +1103,7 @@ interpreted as cited text.)"
 	(real-end end)
 	current  beg
 	e p hend)
-    (if too-big
-	nil
+    (unless too-big
       (save-excursion
 	(save-restriction
 	  (widen)
