@@ -147,9 +147,12 @@ Return its cache buffer."
 	(buf nil))
     (if (< len wl-message-buffer-cache-size)
 	(setq buf (wl-message-buffer-create))
-      (setq buf (wl-message-buffer-cache-buffer-get
-		 (nth (1- len) wl-message-buffer-cache)))
-      (setcdr (nthcdr (- len 2) wl-message-buffer-cache) nil))
+      (let ((entry (nth (1- len) wl-message-buffer-cache)))
+	(if (buffer-live-p
+	     (setq buf (wl-message-buffer-cache-buffer-get entry)))
+	    (setcdr (nthcdr (- len 2) wl-message-buffer-cache) nil)
+	  (setq wl-message-buffer-cache (delq entry wl-message-buffer-cache))
+	  (setq buf (wl-message-buffer-create)))))
     (setq wl-message-buffer-cache
 	  (cons (wl-message-buffer-cache-entry-make key buf)
 		wl-message-buffer-cache))
@@ -366,7 +369,9 @@ Returns non-nil if bottom of message."
 
 (defun wl-message-get-original-buffer ()
   "Get original buffer for current message buffer."
-  wl-message-buffer-original-buffer)
+  (if (buffer-live-p wl-message-buffer-original-buffer)
+      wl-message-buffer-original-buffer
+    (wl-original-message-buffer-get (buffer-name (current-buffer)))))
 
 (defun wl-message-add-buttons-to-body (start end)
   (save-excursion
@@ -418,7 +423,7 @@ Returns non-nil if bottom of message."
 	 message-buf
 	 strategy entity
 	 cache-used
-	 header-end real-fld-num summary-win delim)
+	 summary-win delim)
     (setq buffer-read-only nil)
     (setq cache-used (wl-message-buffer-display
 		      folder number flag force-reload))
@@ -479,6 +484,10 @@ Returns non-nil if bottom of message."
 	  ;; buffer cache is used.
 	  (setq cache-used t)
 	  (with-current-buffer hit
+	    ;; Rewind to the top page
+	    (widen)
+	    (goto-char (point-min))
+	    (ignore-errors (wl-message-narrow-to-page))
 	    (unless (eq wl-message-buffer-cur-flag flag)
 	      (setq read t))))
       ;; delete tail and add new to the top.
