@@ -34,14 +34,6 @@
 (require 'mime-play)
 (require 'elmo)
 
-(eval-when-compile
-  (defalias-maybe 'Meadow-version 'ignore))
-
-(defvar xemacs-betaname)
-(defvar xemacs-codename)
-(defvar enable-multibyte-characters)
-(defvar mule-version)
-
 ;;; Draft
 
 (defalias 'wl-draft-editor-mode 'mime-edit-mode)
@@ -51,7 +43,11 @@
 
 (defun wl-draft-yank-current-message-entity ()
   "Yank currently displayed message entity.
-By setting following-method as yank-content."
+By setting following-method as yank-content.
+
+If region is active, yank region contents instead. \(this feature is available
+if and only if `transient-mark-mode' \(GNU Emacs\) or `zmacs-regions' \(XEmacs\)
+has Non-nil value\)"
   (let ((wl-draft-buffer (current-buffer))
 	(mime-view-following-method-alist
 	 (list (cons 'wl-original-message-mode
@@ -61,26 +57,107 @@ By setting following-method as yank-content."
 		     (function wl-draft-yank-to-draft-buffer)))))
     (if (get-buffer (wl-current-message-buffer))
 	(save-excursion
+	  (set-buffer (wl-current-message-buffer))
 	  (save-restriction
-	    (set-buffer (wl-current-message-buffer))
 	    (widen)
-	    (mime-preview-follow-current-entity))))))
+	    (if (wl-region-exists-p)
+		(wl-mime-preview-follow-current-region)
+	      (mime-preview-follow-current-entity)))))))
+
+;; modified mime-preview-follow-current-entity from mime-view.el
+(defun wl-mime-preview-follow-current-region ()
+  "Write follow message to current region.
+It calls following-method selected from variable
+`mime-preview-following-method-alist'."
+  (interactive)
+  (let ((r-beg (region-beginning))
+	(r-end (region-end))
+	(entity (get-text-property (point-min)
+				   'mime-view-entity)))
+    (let* ((mode (mime-preview-original-major-mode 'recursive))
+	   (new-name
+	    (format "%s-active-region" (buffer-name)))
+	   new-buf
+	   (the-buf (current-buffer))
+	   fields)
+      (save-excursion
+	(set-buffer (setq new-buf (get-buffer-create new-name)))
+	(erase-buffer)
+	(insert ?\n)
+	(insert-buffer-substring the-buf r-beg r-end)
+	(goto-char (point-min))
+	(let ((current-entity
+	       (if (and (eq (mime-entity-media-type entity) 'message)
+			(eq (mime-entity-media-subtype entity) 'rfc822))
+		   (car (mime-entity-children entity))
+		 entity)))
+	  (while (and current-entity
+		      (if (and (eq (mime-entity-media-type
+				    current-entity) 'message)
+			       (eq (mime-entity-media-subtype
+				    current-entity) 'rfc822))
+			  nil
+			(mime-insert-header current-entity fields)
+			t))
+	    (setq fields (std11-collect-field-names)
+		  current-entity (mime-entity-parent current-entity))
+	    ))
+	(let ((rest mime-view-following-required-fields-list)
+	      field-name ret)
+	  (while rest
+	    (setq field-name (car rest))
+	    (or (std11-field-body field-name)
+		(progn
+		  (save-excursion
+		    (set-buffer the-buf)
+		    (let ((entity (when mime-mother-buffer
+				    (set-buffer mime-mother-buffer)
+				    (get-text-property (point)
+						       'mime-view-entity))))
+		      (while (and entity
+				  (null (setq ret (mime-entity-fetch-field
+						   entity field-name))))
+			(setq entity (mime-entity-parent entity)))))
+		  (if ret
+		      (insert (concat field-name ": " ret "\n"))
+		    )))
+	    (setq rest (cdr rest))
+	    ))
+	)
+      (let ((f (cdr (assq mode mime-preview-following-method-alist))))
+	(if (functionp f)
+	    (funcall f new-buf)
+	  (message
+	   "Sorry, following method for %s is not implemented yet."
+	   mode)
+	  ))
+      )))
 
 (defalias 'wl-draft-enclose-digest-region 'mime-edit-enclose-digest-region)
 
 (defun wl-draft-preview-message ()
-  ""
+  "Preview editing message."
   (interactive)
   (let* (recipients-message
+	 (current-point (point))
 	 (config-exec-flag wl-draft-config-exec-flag)
+	 (parent-folder wl-draft-parent-folder)
 	 (mime-display-header-hook 'wl-highlight-headers)
+	 (mime-header-encode-method-alist
+	  (append
+	   '((wl-draft-eword-encode-address-list
+	      .  (To Cc Bcc Resent-To Resent-Cc Resent-Bcc From)))
+	   (if (boundp 'mime-header-encode-method-alist)
+	       (symbol-value 'mime-header-encode-method-alist))))
 	 mime-view-ignored-field-list	; all header.
 	 (mime-edit-translate-buffer-hook
 	  (append
 	   (list
 	    (function
 	     (lambda ()
-	       (let ((wl-draft-config-exec-flag config-exec-flag))
+	       (let ((wl-draft-config-exec-flag config-exec-flag)
+		     (wl-draft-parent-folder parent-folder))
+		 (goto-char current-point)
 		 (run-hooks 'wl-draft-send-hook)
 		 (setq recipients-message
 		       (condition-case err
@@ -109,7 +186,7 @@ By setting following-method as yank-content."
       (when wl-highlight-body-too
 	(wl-highlight-body))
       (run-hooks 'wl-draft-preview-message-hook))
-    (message recipients-message)))
+    (message "%s" recipients-message)))
 
 (defalias 'wl-draft-caesar-region  'mule-caesar-region)
 
@@ -156,6 +233,60 @@ By setting following-method as yank-content."
 (defalias 'wl-message-button-dispatcher-internal
   'mime-button-dispatcher)
 
+(defsubst wl-mime-node-id-to-string (node-id)
+  (if (consp node-id)
+      (mapconcat (function (lambda (num) (format "%s" (1+ num))))
+		 (reverse node-id)
+		 ".")
+    "0"))
+
+(defun wl-message-delete-current-part ()
+  "Delete a part under the cursor from the multipart message."
+  (interactive)
+  (save-restriction
+    (widen)
+    (let* ((entity (get-text-property (point) 'mime-view-entity))
+	   (node-id (mime-entity-node-id entity))
+	   (header-start (mime-buffer-entity-header-start-internal entity))
+	   (body-end (mime-buffer-entity-body-end-internal entity))
+	   (folder (wl-folder-get-elmo-folder wl-message-buffer-cur-folder))
+	   (number wl-message-buffer-cur-number)
+	   (msgid (elmo-message-field folder number 'message-id))
+	   (orig-buf wl-message-buffer-original-buffer))
+      (if (eq (luna-class-name entity) 'mime-elmo-imap-entity)
+	  (error "Please fetch the entire message (by typing 'C-u .') and try again"))
+      (with-current-buffer orig-buf
+	(unless (string-equal
+		 (buffer-string)
+		 (elmo-message-fetch folder number
+				     (elmo-make-fetch-strategy 'entire)))
+	  (error "Buffer content differs from actual message")))
+      (when (and (elmo-folder-writable-p folder)
+		 (buffer-live-p orig-buf)
+		 node-id
+		 (yes-or-no-p
+		  (format "Do you really want to delete part %s? "
+			  (wl-mime-node-id-to-string node-id))))
+	(with-temp-buffer
+	  (insert-buffer orig-buf)
+	  (delete-region header-start body-end)
+	  (goto-char header-start)
+	  (insert "Content-Type: text/plain; charset=US-ASCII\n\n")
+	  (insert "** This part has been removed by Wanderlust **\n\n")
+	  (elmo-folder-append-buffer folder t))
+
+	(elmo-folder-append-messages
+	 (wl-folder-get-elmo-folder wl-trash-folder)
+	 folder (list number) nil)
+	(elmo-folder-delete-messages folder (list number))
+
+	(when (file-exists-p (elmo-cache-get-path msgid))
+	  (delete-file (elmo-cache-get-path msgid)))
+
+	(mime-preview-quit)
+	(wl-summary-toggle-disp-msg 'off)
+	(wl-summary-sync nil "update")))))
+
 ;;; Summary
 (defun wl-summary-burst-subr (message-entity target number)
   ;; returns new number.
@@ -169,7 +300,7 @@ By setting following-method as yank-content."
 			   number))))
 	  ((and (eq (cdr (assq 'type content-type)) 'message)
 		(eq (cdr (assq 'subtype content-type)) 'rfc822))
-	   (message (format "Bursting...%s" (setq number (+ 1 number))))
+	   (message "Bursting...%s" (setq number (+ 1 number)))
 	   (setq entity
 		 (car (mime-entity-children message-entity)))
 	   (with-temp-buffer
@@ -179,17 +310,18 @@ By setting following-method as yank-content."
 	      (mime-entity-fetch-field entity "Message-ID")))))
     number))
 
-(defun wl-summary-burst ()
-  ""
-  (interactive)
+(defun wl-summary-burst (&optional arg)
+  "De-capsulate embedded messages in MIME format.
+With ARG, ask destination folder."
+  (interactive "P")
   (let ((raw-buf (wl-summary-get-original-buffer))
 	(view-buf wl-message-buffer)
 	children message-entity content-type target)
     (save-excursion
       (setq target wl-summary-buffer-elmo-folder)
-      (while (not (elmo-folder-writable-p target))
-	(setq target
-	      (wl-summary-read-folder wl-default-folder "to extract to")))
+      (when (or arg (not (elmo-folder-writable-p target)))
+	(let ((fld (wl-summary-read-folder wl-default-folder "to extract to")))
+	  (setq target (wl-folder-get-elmo-folder fld))))
       (wl-summary-set-message-buffer-or-redisplay)
       (with-current-buffer view-buf
 	(setq message-entity (get-text-property (point-min) 'mime-view-entity)))
@@ -215,11 +347,12 @@ By setting following-method as yank-content."
     (while (file-directory-p filename)
       (setq filename (read-file-name "Please set filename (not directory): "
 				     filename)))
-    (if (file-exists-p filename)
-	(or (yes-or-no-p (format "File %s exists. Save anyway? " filename))
-	    (error "Not saved")))
-    (setq wl-mime-save-directory (file-name-directory filename))
-    (mime-write-entity-content entity filename)))
+    (if (and (file-exists-p filename)
+	     (not (yes-or-no-p (format "File %s exists. Save anyway? "
+				       filename))))
+	(message "Not saved")
+      (setq wl-mime-save-directory (file-name-directory filename))
+      (mime-write-entity-content entity filename))))
 
 ;;; Yet another combine method.
 (defun wl-mime-combine-message/partial-pieces (entity situation)
@@ -298,6 +431,14 @@ By setting following-method as yank-content."
     (elmo-mime-insert-header entity situation)
     (wl-highlight-headers)))
 
+(defun wl-mime-decrypt-application/pgp-encrypted (entity situation)
+  (let ((summary-buffer wl-message-buffer-cur-summary-buffer)
+	(original-buffer wl-message-buffer-original-buffer))
+    (mime-decrypt-application/pgp-encrypted entity situation)
+    (setq wl-message-buffer-cur-summary-buffer summary-buffer)
+    (setq wl-message-buffer-original-buffer original-buffer)))
+   
+
 ;;; Setup methods.
 (defun wl-mime-setup ()
   (set-alist 'mime-preview-quitting-method-alist
@@ -327,6 +468,12 @@ By setting following-method as yank-content."
      (request-partial-message-method . wl-message-request-partial)
      (major-mode . wl-original-message-mode)))
 
+  (ctree-set-calist-strictly
+   'mime-acting-condition
+   '((type . application) (subtype . pgp-encrypted)
+     (method . wl-mime-decrypt-application/pgp-encrypted)
+     (major-mode . wl-original-message-mode)))
+
   (ctree-set-calist-strictly
    'mime-acting-condition
    '((mode . "extract")
