@@ -1,7 +1,8 @@
 ;;; wl-xmas.el --- Wanderlust modules for XEmacsen.
 
-;; Copyright (C) 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
-;; Copyright (C) 2000 Katsumi Yamaoka <yamaoka@jpl.org>
+;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003
+;;  Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2000, 2001, 2002, 2003 Katsumi Yamaoka <yamaoka@jpl.org>
 
 ;; Author: Yuuichi Teranishi <teranisi@gohome.org>
 ;;	Katsumi Yamaoka <yamaoka@jpl.org>
@@ -65,12 +66,12 @@
      wl-folder-prev-entity t "Previous Folder"]
     [wl-folder-check-current-entity
      wl-folder-check-current-entity t "Check Current Folder"]
-;;; [wl-draft
-;;;  wl-draft t "Write a New Message"]
     [wl-folder-sync-current-entity
      wl-folder-sync-current-entity t "Sync Current Folder"]
     [wl-draft
      wl-draft t "Write a New Message"]
+    [wl-folder-goto-draft-folder
+     wl-folder-goto-draft-folder t "Go to Draft Folder"]
     [wl-folder-empty-trash
      wl-folder-empty-trash t "Empty Trash"]
     [wl-exit
@@ -94,7 +95,7 @@
     [wl-summary-mark-as-important
      wl-summary-mark-as-important t "Mark Current Message as Important"]
     [wl-draft
-     wl-draft t "Write a New Message"]
+     wl-summary-write-current-folder t "Write for Current Folder"]
     [wl-summary-reply
      wl-summary-reply t "Reply to Current Message" ]
     [wl-summary-reply-with-citation
@@ -133,6 +134,8 @@
      wl-draft-insert-signature t "Insert Signature"]
     [wl-draft-kill
      wl-draft-kill t "Kill Current Draft"]
+    [wl-draft-save-and-exit
+     wl-draft-save-and-exit t "Save Draft and Exit"]
     )
   "The Draft buffer toolbar.")
 
@@ -199,13 +202,13 @@
   (defsubst wl-xmas-highlight-folder-group-line (glyph text-face numbers)
     (let ((start (match-beginning 1))
 	  (end (match-end 1)))
-      (let (extent)
-	(while (and (setq extent (extent-at start nil nil extent 'at))
-		    (not (and (eq start (extent-start-position extent))
-			      (eq end (extent-end-position extent))
-			      (extent-end-glyph extent)))))
-	(unless extent
-	  (setq extent (make-extent start end)))
+      (let ((extent (or (map-extents
+			 (lambda (extent maparg)
+			   (and (eq start (extent-start-position extent))
+				(eq end (extent-end-position extent))
+				extent))
+			 nil start start nil nil 'end-glyph)
+			(make-extent start end))))
 	(set-extent-properties extent `(end-open t start-closed t invisible t))
 	(set-extent-end-glyph
 	 extent
@@ -264,13 +267,13 @@
 			     (get-text-property (point) 'wl-folder-entity-id)))
 	     (looking-at "[ \t]+\\([^ \t]+\\)"))
 	(let ((start (match-beginning 1)))
-	  (let (extent)
-	    (while (and (setq extent (extent-at start nil nil extent 'at))
-			(not (and (eq start (extent-start-position extent))
-				  (eq start (extent-end-position extent))
-				  (extent-begin-glyph extent)))))
-	    (unless extent
-	      (setq extent (make-extent start start)))
+	  (let ((extent (or (map-extents
+			     (lambda (extent maparg)
+			       (and (eq start (extent-start-position extent))
+				    (eq start (extent-end-position extent))
+				    extent))
+			     nil start start nil nil 'begin-glyph)
+			    (make-extent start start))))
 	    (let (type)
 	      (set-extent-begin-glyph
 	       extent
@@ -444,6 +447,7 @@
 
 (defun wl-message-define-keymap ()
   (let ((keymap (make-sparse-keymap)))
+    (define-key keymap "D" 'wl-message-delete-current-part)
     (define-key keymap "l" 'wl-message-toggle-disp-summary)
     (define-key keymap 'button4 'wl-message-wheel-down)
     (define-key keymap 'button5 'wl-message-wheel-up)
@@ -456,29 +460,35 @@
 
 (defun wl-message-wheel-up (event)
   (interactive "e")
-  (let ((cur-buf (current-buffer))
-	proceed)
-    (save-selected-window
-      (select-window (event-window event))
-      (set-buffer cur-buf)
-      (setq proceed (wl-message-next-page)))
-    (when proceed
-      (if (memq 'shift (event-modifiers event))
-	  (wl-summary-down t)
-	(wl-summary-next t)))))
+  (if (string-match (regexp-quote wl-message-buffer-name)
+		    (regexp-quote (buffer-name)))
+      (wl-message-prev-page)
+    (let ((cur-buf (current-buffer))
+	  proceed)
+      (save-selected-window
+	(select-window (event-window event))
+	(set-buffer cur-buf)
+	(setq proceed (wl-message-next-page)))
+      (when proceed
+	(if (memq 'shift (event-modifiers event))
+	    (wl-summary-down t)
+	  (wl-summary-next t))))))
 
 (defun wl-message-wheel-down (event)
   (interactive "e")
-  (let ((cur-buf (current-buffer))
-	proceed)
-    (save-selected-window
-      (select-window (event-window event))
-      (set-buffer cur-buf)
-      (setq proceed (wl-message-prev-page)))
-    (when proceed
-      (if (memq 'shift (event-modifiers event))
-	  (wl-summary-up t)
-	(wl-summary-prev t)))))
+  (if (string-match (regexp-quote wl-message-buffer-name)
+		    (regexp-quote (buffer-name)))
+      (wl-message-prev-page)
+    (let ((cur-buf (current-buffer))
+	  proceed)
+      (save-selected-window
+	(select-window (event-window event))
+	(set-buffer cur-buf)
+	(setq proceed (wl-message-prev-page)))
+      (when proceed
+	(if (memq 'shift (event-modifiers event))
+	    (wl-summary-up t)
+	  (wl-summary-prev t))))))
 
 (defun wl-draft-overload-menubar ()
   (when (featurep 'menubar)
@@ -487,7 +497,14 @@
     (add-menu-item '("Mail") "Send Message"
 		   'wl-draft-send-and-exit t "Send and Exit")
     (delete-menu-item '("Mail" "Send Mail"))
-    (delete-menu-item '("Mail" "Send and Exit"))))
+    (delete-menu-item '("Mail" "Send and Exit"))
+    (add-menu-item '("Mail") "Preview Message"
+		   'wl-draft-preview-message t "Cancel")
+    (add-menu-item '("Mail") "Save Draft and Exit"
+		   'wl-draft-save-and-exit t "Cancel")
+    (add-menu-item '("Mail") "Kill Current Draft"
+		   'wl-draft-kill t "Cancel")
+    (delete-menu-item '("Mail" "Cancel"))))
 
 (defun wl-draft-mode-setup ()
   (require 'derived)
@@ -512,10 +529,11 @@ Special commands:
   (define-key wl-draft-mode-map "\C-c\C-e" 'wl-draft-config-exec)
   (define-key wl-draft-mode-map "\C-c\C-j" 'wl-template-select)
   (define-key wl-draft-mode-map "\C-c\C-p" 'wl-draft-preview-message)
-  (define-key wl-draft-mode-map "\C-x\C-s" 'wl-draft-save)
+;;   (define-key wl-draft-mode-map "\C-x\C-s" 'wl-draft-save)
   (define-key wl-draft-mode-map "\C-c\C-a" 'wl-addrmgr)
   (define-key wl-draft-mode-map "\C-xk"    'wl-draft-mimic-kill-buffer)
-  (define-key wl-draft-mode-map "\C-c\C-d" 'wl-draft-elide-region))
+  (define-key wl-draft-mode-map "\C-c\C-d" 'wl-draft-elide-region)
+  (define-key wl-draft-mode-map "\C-a" 'wl-draft-beginning-of-line))
 
 (defun wl-draft-overload-functions ()
   (wl-mode-line-buffer-identification)
