@@ -666,9 +666,11 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 			(list wl-folder-buffer-name
 			      wl-plugged-buf-name)
 			"\\|")))
-    (if (and wl-folder-use-frame
-	     (> (length (visible-frame-list)) 1))
-	(delete-frame))
+    (when wl-delete-startup-frame-function
+      (funcall wl-delete-startup-frame-function))
+;;    (if (and wl-folder-use-frame
+;;	     (> (length (visible-frame-list)) 1))
+;;	(delete-frame))
     (setq wl-init nil)
     (remove-hook 'kill-emacs-hook 'wl-save-status)
     t)
@@ -701,29 +703,27 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
     (run-hooks 'wl-init-hook)))
 
 (defun wl-check-environment (no-check-folder)
-  (unless wl-from (error "Please set `wl-from'"))
+  (unless wl-from (error "Please set `wl-from' to your mail address"))
   ;; Message-ID
-  (unless (string-match "[^.]\\.[^.]" (or wl-message-id-domain
-					  (if wl-local-domain
-					      (concat (system-name)
-						      "." wl-local-domain)
-					    (system-name))))
-    (error "Please set `wl-local-domain' to get valid FQDN"))
-  (if (string-match "@" (or wl-message-id-domain
-			    (if wl-local-domain
-				(concat (system-name)
-					"." wl-local-domain)
-			      (system-name))))
-      (error "Please remove `@' from `wl-message-id-domain'"))
-  (if (string= wl-local-domain "localdomain")
-      (error "Please set `wl-local-domain'"))
-  (if (string= wl-message-id-domain "localhost.localdomain")
-      (error "Please set `wl-message-id-domain'"))
+  (when wl-insert-message-id
+    (let ((message-id (funcall wl-message-id-function))
+	  domain)
+      (unless (string-match "^<\\([^@]*\\)@\\([^@]*\\)>$" message-id)
+	(cond
+	 ((string-match "@" wl-message-id-domain)
+	  (error "Please remove `@' from `wl-message-id-domain'"))
+	 (t
+	  (error
+	   "Check around `wl-message-id-function' to get valid Message-ID string"))))
+      (setq domain (match-string 2 message-id))
+      (if (or (not (string-match "[^.]\\.[^.]" domain))
+	      (string= domain "localhost.localdomain"))
+	  (elmo-warning
+	   "Please set `wl-message-id-domain' to get valid Message-ID string."))))
   ;; folders
   (when (not no-check-folder)
     (let ((draft-folder (wl-folder-get-elmo-folder wl-draft-folder))
 	  (queue-folder (wl-folder-get-elmo-folder wl-queue-folder))
-	  (trash-folder (wl-folder-get-elmo-folder wl-trash-folder))
 	  (lost+found-folder (wl-folder-get-elmo-folder
 			      elmo-lost+found-folder)))
       (if (not (elmo-folder-message-file-p draft-folder))
@@ -742,12 +742,6 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	      (elmo-folder-create queue-folder)
 	    (error "Queue Folder is not created")))
       (when (not (eq no-check-folder 'wl-draft))
-	(unless (elmo-folder-exists-p trash-folder)
-	  (if (y-or-n-p
-	       (format "Trash Folder %s does not exist, create it? "
-		       wl-trash-folder))
-	      (elmo-folder-create trash-folder)
-	    (error "Trash Folder is not created")))
 	(unless (elmo-folder-exists-p lost+found-folder)
 	  (elmo-folder-create lost+found-folder)))
       ;; tmp dir
@@ -758,6 +752,48 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 	    (make-directory wl-temporary-file-directory)
 	  (error "Temp directory is not created"))))))
 
+(defconst wl-check-variables-alist
+  '((numberp . elmo-pop3-default-port)
+    (symbolp . elmo-pop3-default-authenticate-type)
+    (numberp . elmo-imap4-default-port)
+    (symbolp . elmo-imap4-default-authenticate-type)
+    (numberp . elmo-nntp-default-port)
+    (numberp . wl-pop-before-smtp-port)
+    (symbolp . wl-pop-before-smtp-authenticate-type)))
+
+(defun wl-check-variables ()
+  (let ((type-variables wl-check-variables-alist)
+	type)
+    (while (setq type (car type-variables))
+      (if (and (eval (cdr type))
+	       (not (funcall (car type)
+			     (eval (cdr type)))))
+	  (error "%s must be %s: %S"
+		 (cdr type)
+		 (substring (format "%s" (car type)) 0 -1)
+		 (eval (cdr type))))
+      (setq type-variables (cdr type-variables)))))
+
+(defun wl-check-variables-2 ()
+  (if (< wl-message-buffer-cache-size 1)
+      (error "`wl-message-buffer-cache-size' must be larger than 0"))
+  (when wl-message-buffer-prefetch-depth
+    (if (not (< wl-message-buffer-prefetch-depth
+		wl-message-buffer-cache-size))
+	(error (concat
+		"`wl-message-buffer-prefetch-depth' must be smaller than "
+		"`wl-message-buffer-cache-size' - 1."))))
+  (let (case-fold-search)
+    (unless (string-match "%T" wl-summary-line-format)
+      (error "`wl-summary-line-format' must contain %%T and %%P"))
+    (unless (string-match "%P" wl-summary-line-format)
+      (error "`wl-summary-line-format' must contain %%T and %%P"))
+    (dolist (pair wl-folder-summary-line-format-alist)
+      (unless (string-match "%T" (cdr pair))
+	(error "Format `%s' must contain %%T and %%P" (cdr pair)))
+      (unless (string-match "%P" (cdr pair))
+	(error "Format `%s' must contain %%T and %%P" (cdr pair))))))
+
 ;;;###autoload
 (defun wl (&optional arg)
   "Start Wanderlust -- Yet Another Message Interface On Emacsen.
@@ -775,14 +811,16 @@ If ARG (prefix argument) is specified, folder checkings are skipped."
     (condition-case obj
 	(progn
 	  (if check
-	      (condition-case nil
-		  (progn
-		    (message "Checking environment...")
-		    (wl-check-environment arg)
-		    (message "Checking environment...done"))
-		(error)
-		(quit)))
-	  (wl-plugged-init (wl-folder arg))
+	      (progn
+		(message "Checking environment...")
+		(wl-check-environment arg)
+		(message "Checking environment...done")
+		(message "Checking type of variables...")
+		(wl-check-variables)
+		(wl-check-variables-2)
+		(message "Checking type of variables...done")))
+	  (let ((inhibit-quit t))
+	    (wl-plugged-init (wl-folder arg)))
 	  (unless arg
 	    (run-hooks 'wl-auto-check-folder-pre-hook)
 	    (wl-folder-auto-check)
@@ -797,6 +835,41 @@ If ARG (prefix argument) is specified, folder checkings are skipped."
 	(kill-buffer demo-buf)))
   (run-hooks 'wl-hook))
 
+(defvar wl-delete-startup-frame-function nil)
+
+;;;###autoload
+(defun wl-other-frame (&optional arg)
+  "Pop up a frame to read messages via Wanderlust."
+  (interactive)
+  (if wl-folder-use-frame
+      (wl arg)
+    (let ((focusing-functions (append '(raise-frame select-frame)
+				      (if (fboundp 'x-focus-frame)
+					  '(x-focus-frame)
+					'(focus-frame))))
+	  (folder (get-buffer wl-folder-buffer-name))
+	  window frame wl-folder-use-frame)
+      (if (and folder
+	       (setq window (get-buffer-window folder t))
+	       (window-live-p window)
+	       (setq frame (window-frame window)))
+	  (progn
+	    (while focusing-functions
+	      (funcall (car focusing-functions) frame)
+	      (setq focusing-functions (cdr focusing-functions)))
+	    (wl arg))
+	(setq frame (make-frame))
+	(while focusing-functions
+	  (funcall (car focusing-functions) frame)
+	  (setq focusing-functions (cdr focusing-functions)))
+	(setq wl-delete-startup-frame-function
+	      `(lambda ()
+		 (setq wl-delete-startup-frame-function nil)
+		 (let ((frame ,frame))
+		   (if (eq (selected-frame) frame)
+		       (delete-frame frame)))))
+	(wl arg)))))
+
 ;; Define some autoload functions WL might use.
 (eval-and-compile
   ;; This little mapcar goes through the list below and marks the
